"""
python -m gumby.cli landmarks examples/vitra/vitra.yml
"""

import click
from .recipe import Recipe

@click.group()
def cli():
    pass

@cli.command()
@click.argument('recipe')
def landmarks(recipe):
    """
    Print the landmarks for the recipe in the YAML file RECIPE.
    """
    from .landmarks import print_landmarks
    recipe_obj = Recipe.load(recipe)
    print_landmarks(recipe_obj.landmarks)

@cli.command()
@click.argument('recipe')
@click.argument('output_path')
def run(recipe, output_path):
    """
    Run the recipe in the YAML file RECIPE and writes it to OUTPUT_PATH.
    """
    recipe_obj = Recipe.load(recipe)
    mesh = recipe_obj.run()
    mesh.write(output_path)

if __name__ == '__main__':
    cli()



class Recipe(object):
    def __init__(self, recipe):
        """
        example recipe:

        mesh: examples/vitra/vitra.obj
        landmarks: examples/vitra/vitra.pp
        segments:
          - ['leg seam', 'knee bottom', 20]
          - ['knee bottom', 'knee top', 10]
          - ['knee top', 'leg top', 10]
          - ['back middle', 'back top', 50]
        """
        self.mesh_path = recipe['mesh']
        self.landmarks_path = recipe['landmarks']
        self.segments = recipe['segments']

    @classmethod
    def load(cls, recipe_path):
        import yaml
        with open(recipe_path, 'r') as f:
            recipe_data = yaml.load(f)
        return Recipe(recipe_data)

    @property
    def source_mesh(self):
        from lace.mesh import Mesh
        mesh = Mesh(filename=self.mesh_path)
        # Fix crash in write_obj.
        del mesh.segm
        return mesh

    @property
    def landmarks(self):
        from lace.serialization import meshlab_pickedpoints
        return meshlab_pickedpoints.load(self.landmarks_path)

    def run(self):
        from .core import stretch_segments_along_y
        return stretch_segments_along_y(
            mesh=self.source_mesh,
            landmarks=self.landmarks,
            segments=self.segments)

def main():
    """
    python -m gumby.recipe
    """
    from .path import relative_to_project
    recipe = Recipe.load(relative_to_project('examples/vitra/vitra.yml'))
    result = recipe.run()
    result.write_obj('stretched.obj')
    result.show()

if __name__ == '__main__':
    main()



def stretch_segment_along_y(mesh, in_range, delta):
    import numpy as np
    height = mesh.v[:, 1]
    minval, maxval = in_range

    verts_in_range, = np.where((height >= minval) & (height <= maxval))
    verts_above, = np.where(height > maxval)

    above_minval = mesh.v[:, 1][verts_in_range] - minval
    mesh.v[:, 1][verts_in_range] += delta / (maxval - minval) * above_minval

    mesh.v[:, 1][verts_above] += delta

def stretch_segments_along_y(mesh, landmarks, segments):
    segments_from_floor_up = sorted(
        segments,
        key=lambda segm: landmarks[segm[0]][1])
    cumulative_delta = 0.
    for bottom, top, delta in segments:
        in_range = (
            landmarks[bottom][1] + cumulative_delta,
            landmarks[top][1] + cumulative_delta)
        stretch_segment_along_y(mesh=mesh, in_range=in_range, delta=delta)
        cumulative_delta += delta
    return mesh

def main():
    """
    python -m gumby.core
    """
    from lace.mesh import Mesh
    from lace.serialization import meshlab_pickedpoints
    from .path import relative_to_project
    from .landmarks import print_landmarks

    original_mesh = Mesh(filename=relative_to_project('examples/vitra/vitra.obj'))
    # Fix crash in write_obj.
    del original_mesh.segm
    mesh = original_mesh.copy()

    landmarks = meshlab_pickedpoints.load(relative_to_project('examples/vitra/vitra.pp'))
    print_landmarks(landmarks, units='cm', precision=1)

    segments = [
        ('leg seam', 'knee bottom', 20),
        ('knee bottom', 'knee top', 10),
        ('knee top', 'leg top', 10),
        ('back middle', 'back top', 50),
    ]
    stretch_segments_along_y(mesh=mesh, landmarks=landmarks, segments=segments)

    mesh.show()
    original_mesh.show()

    mesh.write_obj('stretched.obj')

if __name__ == '__main__':
    main()



def get_height(v):
    return v[1]

def sort_landmarks_by_height(landmark_dict):
    from collections import OrderedDict
    return OrderedDict(sorted(
      landmark_dict.items(),
      key=lambda item: get_height(item[1])))

def print_landmarks(landmark_dict, units='', precision=3):
    sorted_landmarks = sort_landmarks_by_height(landmark_dict)
    value_label = 'height ({})'.format(units) if units else 'height'
    print '{:15} {:>12}'.format('name', value_label)
    print '-' * 36
    for name, vert in ((name, sorted_landmarks[name]) for name in reversed(sorted_landmarks)):
        print '{:15} {:>12.{}f}'.format(name, vert[1], precision)



import importlib
from setuptools import setup, find_packages

with open('README.md') as f:
    readme = f.read()

with open('requirements.txt') as f:
    install_requires = f.read()

setup(
    name='gumby',
    version=importlib.import_module('gumby').__version__,
    description='Stretch polygonal meshes in segments along an axis',
    long_description=readme,
    long_description_content_type='text/markdown',
    author='Metabolize',
    author_email='github@paulmelnikow.com',
    url='https://github.com/metabolize/gumby',
    packages=find_packages(),
    install_requires=install_requires,
    classifiers=[
        'Development Status :: 2 - Pre-Alpha',
        'Intended Audience :: Other Audience',
        'Intended Audience :: Science/Research',
        'License :: OSI Approved :: MIT License',
        'Operating System :: OS Independent',
        'Programming Language :: Python :: 2.7',
        'Topic :: Artistic Software',
        'Topic :: Multimedia :: Graphics :: 3D Modeling',
        'Topic :: Scientific/Engineering :: Visualization'
    ]
)
